--[[ ---------------------------------------------------------------------------

Acheron_Graph: graph reports for Acheron

Acheron_Graph is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Acheron_Graph is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with Acheron_Graph.	If not, see <http://www.gnu.org/licenses/>.

----------------------------------------------------------------------------- ]]

--[[ ---------------------------------------------------------------------------
	 Display parameters for easy tinkering
----------------------------------------------------------------------------- ]]
local GFRAME_YMARGIN = 40
local GFRAME_HEIGHT = 300
local GFRAME_BACKDROP  = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = true, tileSize = 16, edgeSize = 16,
	insets = { left = 3, right = 3, top = 5, bottom = 3 }
}
local GFRAME_ICON_BACKDROP  = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = true, tileSize = 16, edgeSize = 10,
	insets = { left = 3, right = 3, top = 3, bottom = 3 }
}
local GFRAME_XPADDING = 5
local GFRAME_YPADDING = 5
local GFRAME_GRAPH_XPADDING_LEFT = 100
local GFRAME_GRAPH_HEIGHT = 100
local GFRAME_GRAPH_YPADDING_TOP = 3
local GFRAME_GRAPH_YPADDING_BOTTOM = 3
local GFRAME_ICONSET_HEIGHT = 18
local GFRAME_ICONSET_ICONSIZE = 16
local GFRAME_ICONSET_PATH_DAMAGE_DEFAULT = "Interface\\TargetingFrame\\UI-RaidTargetingIcon_7"
local GFRAME_ICONSET_BACKDROP = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
	tile = true,
	tileSize = 16,
}
local GFRAME_ACTORTRACKS_MAXNUMBER = 3
local GFRAME_ACTORTRACK_HEIGHT = 18
local GFRAME_ACTORTRACK_CASTSPELLFRAME_HEIGHT = 16
local GFRAME_ACTORTRACK_CASTSPELLFRAME_ICONSIZE = 16
local GFRAME_ACTORTRACK_PATH_DELAY_DEFAULT = "Interface\\TargetingFrame\\UI-RaidTargetingIcon_7"
local GFRAME_ACTORTRACK_CASTDELAYFRAME_ICONSIZE = 10

--[[ ---------------------------------------------------------------------------
	 Addon and libraries
----------------------------------------------------------------------------- ]]

local L = LibStub("AceLocale-3.0"):GetLocale("Acheron_Graph")
local GUI = LibStub("AceGUI-3.0")
local Graph = LibStub:GetLibrary("LibGraph-2.0")

-- hack to add a AddChildAfter method to AceGUI, delete if it's implemented
GUI.WidgetContainerBase.AddChildAfter = function(self, widget, sibling)
	local siblingIndex = 1
	for _, child in pairs(self.children) do
		if sibling == child then
			break
		end
		siblingIndex = siblingIndex + 1 
	end
	tinsert(self.children, siblingIndex + 1, widget)
	widget:SetParent(self)
	widget.frame:Show()
	self:DoLayout()
end

local Acheron = Acheron
local Acheron_Graph = Acheron_Graph

--[[ ---------------------------------------------------------------------------
	 Hooks
----------------------------------------------------------------------------- ]]
function Acheron_Graph:hook_PopulateEntries(self, name, reportNum)
	Acheron_Graph:CleanupGraphMode()
	Acheron_Graph:PopulateGraph(name, reportNum)
end

--[[ ---------------------------------------------------------------------------
	 GUI work
----------------------------------------------------------------------------- ]]

function Acheron_Graph:CreateFrame()
	-- Adding button to toggle graph visibility to Acheron window
	local bu3 = GUI:Create("Button")
	bu3:SetText(L["Graph"])
	bu3:SetWidth(100)
	bu3:SetCallback("OnClick", function(widget,event,value)
	    -- toggle graph visibility
	    if Acheron_Graph.frameReports:IsShown() then
	    	Acheron_Graph.frameReports:Hide()
	    else
	    	Acheron_Graph.frameReports:Show()
	    end
	end)
	Acheron.frame:AddChildAfter(bu3, Acheron.clearAllButton)
	Acheron.frame.frame:SetFrameStrata("FULLSCREEN") -- Change Acheron frame strata to a lower setting
	
	-- Initializing frames repo
	Acheron_Graph.actorTracks = {}
	Acheron_Graph.iconSets = {}
	Acheron_Graph.icons = {}
	Acheron_Graph.spellCastFrames = {}
	Acheron_Graph.damageFrames = {}
	
	-- Creating graph report window
	local border = CreateFrame("Frame",nil,UIParent)
	self.frameReports = border
	border:SetPoint("TOPLEFT",Acheron.frameReports.border,"TOPLEFT", 0, 0)
	border:SetPoint("BOTTOMRIGHT",Acheron.frameReports.border,"BOTTOMRIGHT", 0,0)
	border:SetBackdrop(GFRAME_BACKDROP)
	border:SetBackdropColor(0.05,0.05,0.05,1)
	border:SetBackdropBorderColor(0.4,0.4,0.4)
	border:SetFrameStrata("FULLSCREEN_DIALOG")
	border:EnableMouse()
	border:SetMovable(true)
	border:SetResizable(true)
	-- Mouse behaviour : moving
	border:SetScript("OnMouseDown", function() 
		if ( ( ( not this.isLocked ) or ( this.isLocked == 0 ) ) and ( arg1 == "LeftButton" ) ) then
			this:StartMoving();
			this.isMoving = true;
		end
	end)
	border:SetScript("OnMouseUp", function() 
		if ( this.isMoving ) then
			this:StopMovingOrSizing();
			this.isMoving = false;
		end
	end)
	
	local content = CreateFrame("Frame",nil,border)
	border.content = content
	content.obj = self
	content:SetPoint("TOPLEFT",border,"TOPLEFT",10,-10)
	content:SetPoint("BOTTOMRIGHT",border,"BOTTOMRIGHT",-10,10)
	
	-- Ensuring that if Acheron is hidden graph frame is hidden
	Acheron.frameReports.content:SetScript("OnHide", function()
		Acheron_Graph.frameReports:Hide()
	end)
	
	border:Hide()
end

-- Hiding frames for reuse
function Acheron_Graph:CleanupGraphMode()
	if self.frameGraph then self.frameGraph:Hide() end
	for _, frame in pairs(self.icons) do
		frame:Hide()
	end
	for _, frame in pairs(self.iconSets) do
		frame:Hide()
	end
	for _, frame in pairs(self.actorTracks) do
		frame:Hide()
	end
	for _, frame in pairs(self.spellCastFrames) do
		frame:Hide()
	end
	for _, frame in pairs(self.damageFrames) do
		frame:Hide()
	end
end

--[[ ---------------------------------------------------------------------------
	 Create and position all frames for graph report
----------------------------------------------------------------------------- ]]
function Acheron_Graph:PopulateGraph(name, reportNum)
	local id = Acheron:GetGUID(tostring(name))
	reportNum = tonumber(reportNum)
	
	local reportSet = Acheron:FilterReport(id, reportNum, Acheron:GetProfileParam("reporttime"))
	if not reportSet[#reportSet] then return end
	local firstTimeStamp = reportSet[1].timeStamp
	local lastTimeStamp = reportSet[#reportSet].timeStamp
	
	---------------------------------------
	-- positionning of icon sets
	local iconSetPositions = {}
	local totalIconSetPositions = 0
	if Acheron:GetProfileParam("showdamage") then
		iconSetPositions["DAMAGE"] = totalIconSetPositions
		totalIconSetPositions = totalIconSetPositions + 1
	end
	if Acheron:GetProfileParam("showhealing") then
		iconSetPositions["HEAL"] = totalIconSetPositions
		totalIconSetPositions = totalIconSetPositions + 1
	end
	if Acheron:GetProfileParam("showbuff") then
		iconSetPositions["BUFF"] = totalIconSetPositions
		totalIconSetPositions = totalIconSetPositions + 1
	end
	if Acheron:GetProfileParam("showdebuff") then
		iconSetPositions["DEBUFF"] = totalIconSetPositions
		totalIconSetPositions = totalIconSetPositions + 1
	end
	for iconSetName, iconSetPosition in pairs(iconSetPositions) do
		local is = self.iconSets[iconSetName]
		if not is then
			is = CreateFrame("Frame","AcheronGraph_IconSet_"..iconSetName,self.frameReports.content)
			is:SetBackdrop(GFRAME_ICONSET_BACKDROP)
			self.iconSets[iconSetName] = is
			
			local label = is:CreateFontString(nil,"BACKGROUND","GameFontHighlightSmall")
			label:SetPoint("TOPLEFT",is,"TOPLEFT",-GFRAME_GRAPH_XPADDING_LEFT-GFRAME_ICONSET_ICONSIZE/2-2,-2)
			label:SetWidth(GFRAME_GRAPH_XPADDING_LEFT)
			label:SetJustifyH("RIGHT")
			label:SetJustifyV("TOP")
			label:SetText(L[iconSetName])
			is.label = label
			
			is:SetHeight(GFRAME_ICONSET_HEIGHT)
		end
		is:SetWidth(self.frameReports.content:GetWidth()-GFRAME_XPADDING*2-GFRAME_GRAPH_XPADDING_LEFT)
		is:ClearAllPoints()
		is:SetPoint("TOPLEFT", self.frameReports.content, "TOPLEFT", 
			GFRAME_XPADDING + GFRAME_GRAPH_XPADDING_LEFT,
			- GFRAME_YPADDING - iconSetPosition * GFRAME_ICONSET_HEIGHT
		)
		is:SetBackdropColor(200/255, 200/255, 200/255, 0.1 + 0.25 * (iconSetPosition % 2))
		
		is:Show()
	end
	
	-------------------------------------------------------------
	-- Graph
	local fg=self.frameGraph 
	if not fg then
		fg = CreateFrame("Frame","Acheron_GraphWindow",self.frameReports.content)
		self.frameGraph = fg
		
		fg:SetHeight(GFRAME_GRAPH_HEIGHT)
		
		local g = Graph:CreateGraphLine("AcheronGraph",fg,"TOPLEFT","TOPLEFT",0,0,fg:GetWidth(),fg:GetHeight())
		fg.graph = g
		
		g:SetGridSpacing(2,20)
		g:SetGridColor({0.5,0.5,0.5,0.5})
		g:SetAxisDrawing(true,true)
		g:SetAxisColor({1.0,1.0,1.0,1.0})
		g:SetAutoScale(false)
	end
	fg:ClearAllPoints()
	fg:SetPoint("TOPLEFT", self.frameReports.content, "TOPLEFT", 
		GFRAME_XPADDING+GFRAME_GRAPH_XPADDING_LEFT, 
		- GFRAME_YPADDING - totalIconSetPositions * GFRAME_ICONSET_HEIGHT - GFRAME_GRAPH_YPADDING_TOP
	)
	fg:SetWidth(self.frameReports.content:GetWidth()-GFRAME_XPADDING*2-GFRAME_GRAPH_XPADDING_LEFT)
		
	fg.graph:SetWidth(fg:GetWidth())
	fg.graph:SetYAxis(0,100)
	fg.graph:SetXAxis(firstTimeStamp-lastTimeStamp,0)
	
	local data = {}
	
	for i,entry in ipairs(reportSet) do
		-- Insert data point in graph
		local dataPoint = {entry.timeStamp-lastTimeStamp, math.max(entry.curHealth / entry.maxHealth * 100, 0)}
		table.insert(data, dataPoint)
		
		-- create and position icon
		local path
		if entry.eventType == "DAMAGE" and entry.amount ~= 0 then -- damages which are not miss/resist/parry/dodge
			if entry.spellId then
				_, _, path = GetSpellInfo(entry.spellId)
			else
				path = GFRAME_ICONSET_PATH_DAMAGE_DEFAULT
			end
		elseif entry.eventType == "BUFF" or entry.eventType == "DEBUFF" or entry.eventType == "HEAL" then -- damages which are not miss/resist/parry/dodge
			_, _, path = GetSpellInfo(entry.spellId)
		end
		if path then
			self:CreateAndPositionIcon(path, entry, firstTimeStamp, lastTimeStamp, name, reportNum)
		end
	end
	
	fg.graph:ResetData()
	fg.graph:AddDataSeries(data,{1.0,0.0,0.0,0.8})
	
	fg:Show()
	
	-------------------------------------------------------------
	-- Creation of interacters tracks
	
	-- Finding latest death entry	
	local interactersSpellLogs = Acheron.combatLogs[id].logs[reportNum].interactersSpellLogs
	
	local currentTrackCount = 1 
	for interacterGUID, interacterSpellLog in pairs(interactersSpellLogs) do
		local at = self.actorTracks[currentTrackCount]
		if not at then
			at = CreateFrame("Frame","Acheron_ActorTrack_"..currentTrackCount, self.frameReports.content)
			at:SetBackdrop(GFRAME_ICONSET_BACKDROP)
			self.actorTracks[currentTrackCount] = at
			
			local label = at:CreateFontString(nil,"BACKGROUND","GameFontHighlightSmall")
			label:SetPoint("TOPLEFT",at,"TOPLEFT",-GFRAME_GRAPH_XPADDING_LEFT-GFRAME_ICONSET_ICONSIZE/2-2,-2)
			label:SetWidth(GFRAME_GRAPH_XPADDING_LEFT)
			label:SetJustifyH("RIGHT")
			label:SetJustifyV("TOP")
			at.label = label
			
			at:SetHeight(GFRAME_ACTORTRACK_HEIGHT)
		end
		at:SetWidth(self.frameReports.content:GetWidth()-GFRAME_XPADDING*2-GFRAME_GRAPH_XPADDING_LEFT)
		at:ClearAllPoints()
		at:SetPoint("TOPLEFT", self.frameReports.content, "TOPLEFT", 
			GFRAME_XPADDING + GFRAME_GRAPH_XPADDING_LEFT,
			- GFRAME_YPADDING - totalIconSetPositions * GFRAME_ICONSET_HEIGHT - GFRAME_GRAPH_YPADDING_TOP - GFRAME_GRAPH_HEIGHT - GFRAME_GRAPH_YPADDING_BOTTOM - (currentTrackCount-1) * GFRAME_ACTORTRACK_HEIGHT
		)
		at:SetBackdropColor(200/255, 200/255, 200/255, 0.1 + 0.25 * ((currentTrackCount + 1) % 2))
		at.label:SetText(Acheron.guidNames[interacterGUID])
		at:Show()
		
		for i = interacterSpellLog.first, interacterSpellLog.last do
			local entry = interacterSpellLog[i]
			if entry.eventType == "START" then
				self:CreateAndPositionSpellCastFrame(entry, at, firstTimeStamp, lastTimeStamp, id)
			elseif entry.eventType == "SUCCESS" then
				self:CreateAndPositionSpellSuccessFrame(entry, at, firstTimeStamp, lastTimeStamp, id)
			elseif entry.eventType == "DAMAGE" then
				self:CreateAndPositionDamageFrame(entry, at, firstTimeStamp, lastTimeStamp)
			end
		end
		
		if currentTrackCount >= GFRAME_ACTORTRACKS_MAXNUMBER then break end
		currentTrackCount = currentTrackCount + 1
	end
end

--[[ ---------------------------------------------------------------------------
	 Create and position icons for events that occured to reported unit
----------------------------------------------------------------------------- ]]

local eventWidget_OnEnter = Acheron.eventWidget_OnEnter
local eventWidget_OnLeave = Acheron.eventWidget_OnLeave
local eventWidget_OnMouseDown = Acheron.eventWidget_OnMouseDown

function Acheron_Graph:CreateAndPositionIcon(path, entry, firstTimeStamp, lastTimeStamp, name, reportNum)
	local is = self.iconSets[entry.eventType]
	
	-- Find first icon available otherwise creates one
	local frame
	for _, iconFrame in pairs(self.icons) do
		if not iconFrame:IsShown() then
			 frame = iconFrame
			 break
		end
	end
	if not frame then
		frame = CreateFrame("Button",nil,is)
		table.insert(self.icons, frame)
		frame:SetWidth(GFRAME_ICONSET_ICONSIZE)
		frame:SetHeight(GFRAME_ICONSET_ICONSIZE)
		frame:EnableMouse(true)
		frame:SetScript("OnLeave", eventWidget_OnLeave)
		frame:SetScript("OnEnter", eventWidget_OnEnter)
		frame:SetScript("OnMouseDown", eventWidget_OnMouseDown)
		
		local image = frame:CreateTexture(nil,"BACKGROUND")
		frame.image = image
		image:SetWidth(GFRAME_ICONSET_ICONSIZE)
		image:SetHeight(GFRAME_ICONSET_ICONSIZE)
		image:SetPoint("CENTER",frame,"CENTER",0,0)
		image:Show()
		
		local highlight = frame:CreateTexture(nil,"OVERLAY")
		frame.highlight = highlight
		highlight:SetAllPoints(image)
		highlight:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-Tab-Highlight")
		highlight:SetTexCoord(0,1,0.23,0.77)
		highlight:SetBlendMode("ADD")
		highlight:Hide()
	end
	frame:Show()
	
	frame.obj = {}
	frame.obj.userdata = {}-- necessary for correct reuse of Acheron widget events	
	frame.obj.userdata.entry = entry
	frame.obj.userdata.seconds = lastTimeStamp - entry.timeStamp
	frame.obj.userdata.name = name
	frame.obj.userdata.reportNum = reportNum
	if entry.eventType == "BUFF" or entry.eventType == "DEBUFF" then
		frame:SetBackdrop(GFRAME_ICON_BACKDROP)
		frame:SetBackdropColor(0,0,0,0)
		if string.sub(entry.action, 1, 1) == "+" then
			if entry.eventType == "BUFF" then
				frame:SetBackdropBorderColor(0, 0, 1, 1)
			else
				frame:SetBackdropBorderColor(1, 0, 0, 1)
			end
		else
			if entry.eventType == "DEBUFF" then
				frame:SetBackdropBorderColor(0, 0, 1, 1)
			else
				frame:SetBackdropBorderColor(1, 0, 0, 1)
			end
		end
	else
		frame:SetBackdrop(nil)
	end 
	
	frame:SetPoint("TOPLEFT", self.iconSets[entry.eventType], "TOPLEFT", 
		math.max(is:GetWidth() * (entry.timeStamp-firstTimeStamp) / (lastTimeStamp-firstTimeStamp) - GFRAME_ICONSET_ICONSIZE/2, 0),
		- (GFRAME_ICONSET_HEIGHT-GFRAME_ICONSET_ICONSIZE)/2
	)
	
	frame.image:SetTexture(path)
end

--[[ ---------------------------------------------------------------------------
	 Create and position frames for events befalling units who
	 interacted with reported unit
----------------------------------------------------------------------------- ]]

-- Handle widget events
local function eventSpellCastWidget_OnEnter(self)
	GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
	GameTooltip:ClearLines()
	
	local entry = self.entry
	
	if entry and entry.msg then
		GameTooltip:AddLine(format("|cffffffff%s|r", entry.msg))
	end
	-- Display cast information if relevant
	if entry.eventType == "START" then
		-- cast time info
		GameTooltip:AddLine(format("|cffffffffCast time : %.2f seconds (%s pushbacks)|r",
			(entry.endTime - entry.startTime)/1000,
			entry.pushBacks	
		))
		-- target info if known
		local targets = ""
		for targetGUID, _ in pairs(entry.targetsGUID) do
			if Acheron.guidNames[targetGUID] then
				targets = targets .. " " .. Acheron.guidNames[targetGUID]
			end
		end
		if #targets>0 then
			GameTooltip:AddLine(format("|cffffffffTargets : %s|r",
				targets
			))
		end
		-- if spell cast failed, display why
		if entry.failType then
			GameTooltip:AddLine(format("|cffff8800Cast failed : %s|r",
				entry.failType
			))
		end
	end
	
	GameTooltip:Show()
	self.highlight:Show()
end

local function eventSpellCastWidget_OnLeave(self)
	GameTooltip:Hide()
	self.highlight:Hide()
end

function Acheron_Graph:CreateAndPositionDamageFrame(entry, actorTrack, firstTimeStamp, lastTimeStamp)
	 -- do not display delays before first death report event and after death event
	local frameOffsetX = actorTrack:GetWidth() * (entry.timeStamp-firstTimeStamp) / (lastTimeStamp-firstTimeStamp)
	if frameOffsetX <= 0 or frameOffsetX >= actorTrack:GetWidth() then return end
	
	-- Find first spellcast delay frame available otherwise creates one
	local frame	
	for _, damageFrame in pairs(self.damageFrames) do
		if not damageFrame:IsShown() then
			 frame = damageFrame
			 break
		end
	end
	if not frame then
		frame = CreateFrame("Frame",nil,actorTrack)
		table.insert(self.damageFrames, frame)
		frame:SetHeight(GFRAME_ACTORTRACK_CASTDELAYFRAME_ICONSIZE)
		frame:SetWidth(GFRAME_ACTORTRACK_CASTDELAYFRAME_ICONSIZE)
		frame:SetFrameLevel(2)
		frame:EnableMouse(true)
		frame:SetScript("OnLeave", eventSpellCastWidget_OnLeave)
		frame:SetScript("OnEnter", eventSpellCastWidget_OnEnter)
		
		local image = frame:CreateTexture(nil,"BACKGROUND")
		frame.image = image
		image:SetWidth(GFRAME_ACTORTRACK_CASTDELAYFRAME_ICONSIZE)
		image:SetHeight(GFRAME_ACTORTRACK_CASTDELAYFRAME_ICONSIZE)
		image:SetPoint("TOPLEFT",frame,"TOPLEFT",0,0)
		image:SetTexture(GFRAME_ACTORTRACK_PATH_DELAY_DEFAULT)
		image:Show()
		
		local highlight = frame:CreateTexture(nil,"OVERLAY")
		frame.highlight = highlight
		highlight:SetAllPoints(image)
		highlight:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-Tab-Highlight")
		highlight:SetTexCoord(0,1,0.23,0.77)
		highlight:SetBlendMode("ADD")
		highlight:Hide()
	end
	
	-- Sets meaningful frame attributes
	frame:ClearAllPoints()
	frame:SetPoint("TOPLEFT", actorTrack, "TOPLEFT", 
		frameOffsetX,
		- (GFRAME_ACTORTRACK_HEIGHT - GFRAME_ACTORTRACK_CASTDELAYFRAME_ICONSIZE)/2
	)
	frame:Show()
	
	frame.entry = entry	
end

function Acheron_Graph:CreateAndPositionSpellCastFrame(entry, actorTrack, firstTimeStamp, lastTimeStamp, deathGUID)
	 -- do not display casts before first death report event and after death event
	local frameOffsetX = actorTrack:GetWidth() * (entry.timeStamp-firstTimeStamp) / (lastTimeStamp-firstTimeStamp)
	local frameFarEdgeX = frameOffsetX + (entry.endTime-entry.startTime)/1000 * actorTrack:GetWidth() / (lastTimeStamp - firstTimeStamp)
	if frameFarEdgeX <= 0 or frameOffsetX >= actorTrack:GetWidth() then return end
	
	-- Find first spellcast frame available otherwise creates one
	local frame	
	for _, spellCastFrame in pairs(self.spellCastFrames) do
		if not spellCastFrame:IsShown() then
			 frame = spellCastFrame
			 break
		end
	end
	if not frame then
		frame = CreateFrame("Frame",nil,actorTrack)
		table.insert(self.spellCastFrames, frame)
		frame:SetHeight(GFRAME_ACTORTRACK_CASTSPELLFRAME_HEIGHT)
		frame:SetBackdrop(GFRAME_ICONSET_BACKDROP)
		
		local icon = CreateFrame("Button",nil,frame)
		icon:SetWidth(GFRAME_ACTORTRACK_CASTSPELLFRAME_ICONSIZE)
		icon:SetHeight(GFRAME_ACTORTRACK_CASTSPELLFRAME_ICONSIZE)
		icon:SetPoint("TOPLEFT",frame,"TOPLEFT",0,0)
		icon:SetBackdrop(GFRAME_ICON_BACKDROP)
		icon:SetBackdropColor(0,0,0,0)
		icon:EnableMouse(true)
		icon:SetScript("OnLeave", eventSpellCastWidget_OnLeave)
		icon:SetScript("OnEnter", eventSpellCastWidget_OnEnter)
		icon:Show()
		frame.icon = icon
		
		local image = icon:CreateTexture(nil,"BACKGROUND")
		icon.image = image
		image:SetWidth(GFRAME_ACTORTRACK_CASTSPELLFRAME_ICONSIZE)
		image:SetHeight(GFRAME_ACTORTRACK_CASTSPELLFRAME_ICONSIZE)
		image:SetPoint("TOPLEFT",icon,"TOPLEFT",0,0)
		image:Show()
		
		local highlight = icon:CreateTexture(nil,"OVERLAY")
		icon.highlight = highlight
		highlight:SetAllPoints(image)
		highlight:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-Tab-Highlight")
		highlight:SetTexCoord(0,1,0.23,0.77)
		highlight:SetBlendMode("ADD")
		highlight:Hide()
	end
	
	-- Sets meaningful frame attributes
	frame:ClearAllPoints()
	frame:SetPoint("TOPLEFT", actorTrack, "TOPLEFT", 
		math.max(frameOffsetX, 0),
		- (GFRAME_ACTORTRACK_HEIGHT - GFRAME_ACTORTRACK_CASTSPELLFRAME_HEIGHT)/2
	)
	frame:SetWidth(
		math.min(frameFarEdgeX, actorTrack:GetWidth()) - math.max(frameOffsetX, 0)
	)
	if frame:GetWidth() == 0 then frame:SetWidth(1) end -- Avoid frame not showing up if spell is instacast
	-- color icon border green if target is processed unit
	if entry.targetsGUID[deathGUID] then
		frame.icon:SetBackdropBorderColor(0, 1, 0, 1)
	else
		-- color icon border orange if spellcast is failed 
		if entry.failType then
			frame.icon:SetBackdropBorderColor(1, 0.5, 0, 1)
		else
			frame.icon:SetBackdropBorderColor(0, 0, 0, 0)
		end
	end
	-- color cast frame orange if spellcast is failed 
	if entry.failType then
		frame:SetBackdropColor(1, 0.5, 0, 0.75)
	else
		frame:SetBackdropColor(1, 1, 0, 0.75)
	end
	
	
	if entry.spellId then
		local _,_,path = GetSpellInfo(entry.spellId)
		frame.icon.image:SetTexture(path)
		frame.icon.image:Show()
	else
		frame.icon.image:Hide()
	end
	frame:Show()
	
	frame.icon.entry = entry
	
end